package cn.newfeifan.mall.sale.service.shop;

import javax.validation.*;

import cn.newfeifan.mall.module.system.controller.admin.user.vo.user.UserRespVO;
import cn.newfeifan.mall.sale.controller.admin.shop.vo.ShopRespVO;
import cn.newfeifan.mall.sale.controller.admin.shop.vo.ShopSaveReqVO;
import cn.newfeifan.mall.sale.dal.dataobject.merchant.MerchantDO;
import cn.newfeifan.mall.sale.dal.dataobject.shop.ShopDO;
import cn.newfeifan.mall.framework.common.pojo.PageResult;
import cn.newfeifan.mall.sale.controller.admin.shop.vo.ShopPageReqVO;
import cn.newfeifan.mall.sale.dal.redis.ShopMessage;

import java.util.List;

/**
 * 店铺 Service 接口
 *
 * @author 非繁人
 */
public interface ShopService {

    /**
     * 创建店铺
     *
     * @param createReqVO 创建信息
     * @return 编号
     */
    Long createShop(@Valid ShopSaveReqVO createReqVO);

    /**
     * 创建店铺
     *
     * @param createReqVO 创建信息
     * @return 编号
     */
    Long createShop( MerchantDO createReqVO);


    /**
     * 更新店铺
     *
     * @param updateReqVO 更新信息
     */
    void updateShop(@Valid ShopSaveReqVO updateReqVO);

    /**
     * 删除店铺
     *
     * @param id 编号
     */
    void deleteShop(Long id);

    /**
     * 获得店铺
     *
     * @param id 编号
     * @return 店铺
     */
    ShopDO getShop(Long id);

    /**
     * 获得店铺分页
     *
     * @param pageReqVO 分页查询
     * @return 店铺分页
     */
    PageResult<ShopDO> getShopPage(ShopPageReqVO pageReqVO);


    /**
     * 获取商户下的所有店铺
     * @param merId
     * @return
     */
    List<ShopRespVO> getMerchantAllShop(Long merId);


    /**
     * 获取店铺下的所有用户
     * @param id
     * @return
     */
    List<UserRespVO> getAllUser(Long id);

    /**
     * 根据店铺列表获取所有的用户
     * @param shopIds
     * @return
     */
    List<UserRespVO> getAllUser(List<Long> shopIds);

    /**
     * 刷新
     *
     * @param trim
     * @param loginUserId
     * @param shopId
     */
    void changeShop(Long loginUserId, Long shopId);

    ShopMessage getCurrentShop(Long loginUserId);

    List<ShopRespVO> getAllShopByUserId(Long loginUserId);

    List<Long> getShopByName(String shopName);
}